package omnigkgui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.Vector;

import omnigkgui.gkcmd.*;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huang, Hsiao-Mei
 * @version      1.0
 */

public class AppFrame extends JFrame implements GkCmdIDs {
    JPanel contentPane;
    JToolBar toolBar = new JToolBar();
    JLabel statusBar = new JLabel();
    BorderLayout borderLayout = new BorderLayout();
    JTabbedPane tabbedPane = new JTabbedPane();

    ImageIcon mainImage;
    // GK-list internal storage data structure
    protected Vector gkList;

    static final String CONNECT = "connect";
    static final String DISCONNECT = "disconnect";
    static final String REFRESHALL = "refreshall";
    static final String CLEARBROWSER = "clearbrowser";
    static final String BUTTONLISTMODE = "buttonmode";
    static final String TREELISTMODE = "treemode";
    static final String CLOSESESSION = "closesession";
    static final String TOTALUSETIME = "totalusetime";
    static final String EXIT = "exit";
    static final String ABOUT = "about";
    static final String MONITOR = "monitor";


    private JMenuItem itmViewBtn = null;
    private JMenuItem itmViewTree = null;

    /** protected or private or some use-restricted interfaces */
    Color btnBgColor = new Color(25, 69, 165);
    /** Class defining the IP-Address interactive dialog box */
    class InputIPDlg extends JDialog implements ActionListener {
        static final String OK = "  ok  ";
        static final String CANCEL = "cancel";

        // GUI components
        protected JLabel dlgLabel;
        protected JTextField dlgIPField=null;
        protected JTextField dlgPortField=null;
        protected JButton okBtn;
        protected JButton cancelBtn;


        /** Construct the InputDlg object */
        public InputIPDlg(Frame parent, String title) {
            super(parent, title, true);
            // set dialog inside the parent's window
            //setSize(300, 150);
            setLocationRelativeTo(parent);
            // add graphical components
            dlgLabel = new JLabel(" The GK's IP & port : ");
            dlgIPField = new JTextField();
            dlgIPField.setColumns(16);
            dlgIPField.requestFocus();
            dlgPortField = new JTextField("7000");
            dlgPortField.setColumns(6);
            okBtn = new JButton(" OK ");
            okBtn.setActionCommand(OK);
            okBtn.addActionListener(this);
            cancelBtn = new JButton(" CANCEL ");
            cancelBtn.setActionCommand(CANCEL);
            cancelBtn.addActionListener(this);
            this.addKeyListener(
              new KeyListener(){
                public void keyPressed(KeyEvent e){
                  if( e.getKeyChar() == e.VK_ENTER){
                  doOKBtn();
                  }
                }
                public void keyReleased(KeyEvent e){
                }
                public void keyTyped(KeyEvent e){
                }
              }
            );
            GridBagLayout dlgLayout = new GridBagLayout();
            GridBagConstraints constraints = new GridBagConstraints();
            getContentPane().setLayout(dlgLayout);

            // set constraints and add the dialog label
            constraints.fill = GridBagConstraints.NONE;
            constraints.anchor = GridBagConstraints.NORTHWEST;
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = 1;
            constraints.insets = new Insets(5, 15, 0, 15);
            getContentPane().add(dlgLabel, constraints);

            // set constraints and add the dialog IP field
            constraints.insets = new Insets(10, 15, 5, 15);
            getContentPane().add(dlgIPField, constraints);

            // set constraints and add the dialog Port field
            constraints.insets = new Insets(5, 15, 10, 15);
            getContentPane().add(dlgPortField, constraints);

            // set constraints and add the OK button and CANCEL button
            constraints.anchor = GridBagConstraints.CENTER;
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.insets = new Insets(10, 15, 10, 15);
            getContentPane().add(okBtn, constraints);

            constraints.gridx = GridBagConstraints.RELATIVE;
            constraints.insets = new Insets(10, 15, 10, 15);
            getContentPane().add(cancelBtn, constraints);

            pack();
            setVisible(true);
        }

        // OK button action
        public void actionPerformed(ActionEvent e) {
            enterKey(e);
        }
        // Make the OK button works
        public void enterKey(ActionEvent e){
           String cmd = e.getActionCommand();
            if (cmd == CANCEL) {
                dispose();
            } else if (cmd == OK) {
                doOKBtn();
            }
        }
          public void doOKBtn(){
          String ip = dlgIPField.getText();
          int port = (new Integer(dlgPortField.getText())).intValue();
          GateKeeper newTab = new GateKeeper(ip, port, tabbedPane);
          newTab.jbInit();
          tabbedPane.addTab(ip, null, newTab);
          tabbedPane.setSelectedComponent(newTab);
          gkList.addElement(newTab);
          dispose();
        }

    } // end of InputIPDlg class definitation

    class FindEPDlg extends JDialog {
      private JTextField searchTXT = null;

      protected JLabel label = new JLabel("Find An EndPoint:");
        final JButton find = new JButton("  Find  ");
        final JButton more = new JButton("  More  ");

    public FindEPDlg(){
        searchTXT= new JTextField();
        searchTXT.setColumns(16);

        find.setBackground(Color.gray);
        // set the event-handler for "find" button
        //find.setIcon(image4);
        find.setBorder(new LineBorder(Color.white, 2, true));
        find.setBackground(SystemColor.info);
        find.setHorizontalAlignment(SwingConstants.LEFT);
        find.setToolTipText("Find a H323 endpoint to see if it has registered on gatekeeper");
        find.addActionListener(
            // anonymous inner class
            new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                    String toFind = searchTXT.getText();
                    searchTXT.setText("");
                    if ((toFind != null) && !toFind.equals(""))
                        findEPBtn_actionPerformed(e, toFind);
                }
            }
        );

        // set the event-handler for "find" button
        //more.setIcon(image4);
        more.setBorder(new LineBorder(Color.white, 2, true));
        more.setBackground(SystemColor.info);
        more.setHorizontalAlignment(SwingConstants.LEFT);
        more.setToolTipText("Find detail H323 endpoint's info if it has registered on gatekeeper");
        more.addActionListener(
            // anonymous inner class
            new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                    String toFind = searchTXT.getText();
                    searchTXT.setText("");
                    if ((toFind != null) && !toFind.equals(""))
                        findEPVerboseBtn_actionPerformed(e, toFind);
                }
            }
        );
        JPanel btnPanel = new JPanel();
        btnPanel.setLayout(new BorderLayout());
        btnPanel.add(find, BorderLayout.WEST);
        btnPanel.add(more, BorderLayout.EAST);
        /**
        this.getContentPane().setLayout(new BorderLayout());
        this.getContentPane().add(label, BorderLayout.NORTH);
        this.getContentPane().add(searchTXT, BorderLayout.CENTER);
        this.getContentPane().add(btnPanel, BorderLayout.SOUTH);
*/

        GridBagLayout dlgLayout = new GridBagLayout();
            GridBagConstraints constraints = new GridBagConstraints();
            getContentPane().setLayout(dlgLayout);

            // set constraints and add the dialog label
            constraints.fill = GridBagConstraints.NONE;
            constraints.anchor = GridBagConstraints.NORTHWEST;
            constraints.gridwidth = GridBagConstraints.REMAINDER;
            constraints.gridheight = 1;
            constraints.insets = new Insets(5, 15, 0, 15);
            getContentPane().add(label, constraints);

            // set constraints and add the dialog IP field
            constraints.insets = new Insets(10, 15, 5, 15);
            getContentPane().add(searchTXT, constraints);

            // set constraints and add the Find button and More button
            constraints.anchor = GridBagConstraints.CENTER;
            constraints.gridwidth = 1;
            constraints.gridheight = 1;
            constraints.insets = new Insets(10, 15, 10, 15);
            getContentPane().add(find, constraints);

            constraints.gridx = GridBagConstraints.RELATIVE;
            constraints.insets = new Insets(10, 15, 10, 15);
            getContentPane().add(more, constraints);

            pack();
            setVisible(true);

}
    protected void findEPBtn_actionPerformed(ActionEvent e, String str) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            // show the message on the Status-Logging pane, text mode, use listener
/*
            String comment = "\n" +
                        "****************************************************************************\n" +
                        "  Find a H323 endpoint to see if it has registered on gatekeeper\n" +
                        "****************************************************************************\n";
            thisTab.listener.transferCommand(comment, "Find " + str + "\n");
*/
            thisTab.commander.transferCommand(GkCmdIDs.GKCmdFind, str);
        }
    }
    protected void findEPVerboseBtn_actionPerformed(ActionEvent e, String str) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            // show the message on the Status-Logging pane, text mode, use listener
/*
            String comment = "\n" +
                        "*******************************************************************************\n" +
                        "  Find detail H323 endpoint's info if it has registered on gatekeeper\n" +
                        "*******************************************************************************\n";
            thisTab.listener.transferCommand(comment, "FindVerbose " + str + "\n");
*/
            thisTab.commander.transferCommand(GkCmdIDs.GKCmdFindVerbose, str);
        }
    }

    }//End of FindEPElg class

    /** Construct the AppFrame object */
    public AppFrame() {
        gkList = new Vector();
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        InputIPDlg ipDlg = new InputIPDlg(this, "Mornitor the GK");
        ipDlg.dlgIPField.requestFocus();
        ipDlg.setLocation(300,300);
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
        //mainFrame = this;
    }

    /** About action performed */
    public void buttonAbout_actionPerformed(ActionEvent e) {
        AppFrame_AboutBox dlg = new AppFrame_AboutBox(this);
        Dimension dlgSize = dlg.getPreferredSize();
        Dimension frmSize = getSize();
        Point loc = getLocation();
        dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
        dlg.setModal(true);
        dlg.show();
    }

    /** Exit action performed */
    public void buttonExit_actionPerformed(ActionEvent e) {
        System.exit(0);
    }
    /** Monitor GK action performed */
    public void buttonMonitor_actionPerformed(ActionEvent e) {
        InputIPDlg ipDlg = new InputIPDlg(this, "Mornitor the GK");
    }
    protected void cachedEPBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            String comment = "\n" +
                       "*********************************************************************************\n" +
                       "  Print Current Cached EndPoints, Come from Neighbor's GK\n" +
                       "*********************************************************************************\n";
            thisTab.listener.transferCommand(comment, "PrintAllCached\n");
        }
    }
    protected void cachedEPVerboseBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            String comment = "\n" +
                       "*****************************************************************************************\n" +
                       "  Print Detail Current Cached EndPoints, Come from Neighbor's GK\n" +
                       "*****************************************************************************************\n";
            thisTab.listener.transferCommand(comment, "PrintAllCached true\n");
        }
    }
    protected void currentCallBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            String comment = "\n" +
                       "*******************************\n" +
                       "  Print Current Calls\n" +
                       "*******************************\n";
            thisTab.listener.transferCommand(comment, "PrintCurrentCalls\n");
        }
    }
    protected void currentCallVerboseBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            String comment = "\n" +
                       "****************************************************\n" +
                       "  Print Current Calls in Detail Format\n" +
                       "****************************************************\n";
            thisTab.listener.transferCommand(comment, "PrintCurrentCallsVerbose\n");
        }
    }
    protected void currentEPBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            // show the message on the Status-Logging pane, text mode, use listener
            String comment = "\n" +
                        "************************************************************\n" +
                        "  Print Current Registered H323 EndPoints\n" +
                        "************************************************************\n";
            thisTab.listener.transferCommand(comment, "PrintAllRegistrations\n");

            // show the message on another message box, GUI mode, use commander
//          thisTab.commander.transferCommand(comment, "PrintAllRegistrations\n");
        }
    }
    protected void currentEPVerboseBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            // show the message on the Status-Logging pane, text mode, use listener
            String comment = "\n" +
                        "********************************************************************************\n" +
                        "  Print Current Registered H323 EndPoints in Detail Format\n" +
                        "********************************************************************************\n";
            thisTab.listener.transferCommand(comment, "PrintAllRegistrationsVerbose\n");
        }
    }
    /** GK Command action-preform functions */
    protected void findEPBtn_actionPerformed(ActionEvent e, String str) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            // show the message on the Status-Logging pane, text mode, use listener
/*
            String comment = "\n" +
                        "****************************************************************************\n" +
                        "  Find a H323 endpoint to see if it has registered on gatekeeper\n" +
                        "****************************************************************************\n";
            thisTab.listener.transferCommand(comment, "Find " + str + "\n");
*/
            thisTab.commander.transferCommand(GkCmdIDs.GKCmdFind, str);
        }
    }
    protected void findEPVerboseBtn_actionPerformed(ActionEvent e, String str) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            // show the message on the Status-Logging pane, text mode, use listener
/*
            String comment = "\n" +
                        "*******************************************************************************\n" +
                        "  Find detail H323 endpoint's info if it has registered on gatekeeper\n" +
                        "*******************************************************************************\n";
            thisTab.listener.transferCommand(comment, "FindVerbose " + str + "\n");
*/
            thisTab.commander.transferCommand(GkCmdIDs.GKCmdFindVerbose, str);
        }
    }
    protected void gkCmdConsoleDebugBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            DebugConsoleFrame debug= new DebugConsoleFrame(this, " GK Debug Console ");
            Dimension frmSize = getSize();
            Dimension dbgSize = new Dimension((int)(frmSize.width * 0.7), (int)(frmSize.height * 0.7));
            Point loc = getLocation();
            debug.setLocation((frmSize.width - dbgSize.width) / 2 + loc.x, (frmSize.height - dbgSize.height) / 2 + loc.y);
            debug.setSize(dbgSize);
            debug.show();

            // debug console network initializing
            debug.setGKIPAddress(thisTab.getHostName(), thisTab.getPort());
            new Thread(debug).start();
        }
    }
/*
    protected void gkCmdWhoBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            String comment = "\n" +
                       "****************************************************************\n" +
                       "  GK Command for Print \"Who am I\" Message\n" +
                       "****************************************************************\n";
            thisTab.listener.transferCommand(comment, "Who\n");
        }
    }
*/

    protected void gkCmdFrameDebugBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
            thisTab.commander.transferCommand(GkCmdIDs.GKCmdDebug, null);
        }
    }
    protected void gkCmdHelpBtn_actionPerformed(ActionEvent e) {
        GateKeeper thisTab = (GateKeeper) tabbedPane.getSelectedComponent();
        if (thisTab != null) {
/*
            String comment = "\n" +
                        "***************************************************\n" +
                        "  GK Commands Help Menu for Use\n" +
                        "***************************************************\n";
            thisTab.listener.transferCommand(comment, "Help\n");
*/
            thisTab.commander.transferCommand(GkCmdIDs.GKCmdHelp, null);
        }
    }

    //add menu in the GUI

    /** Component initialization */
    private void jbInit() throws Exception {
        mainImage = new ImageIcon(omnigkgui.AppFrame.class.getResource("images/prog.gif"));
        this.setIconImage(mainImage.getImage());

        contentPane = (JPanel) this.getContentPane();
        contentPane.setLayout(borderLayout);
        this.setSize(new Dimension(800, 600));
        this.setTitle("Citron's Omni-GK GUI Ver. 1.0");
        statusBar.setText(" ");



        tabbedPane.setBackground(new Color(160, 135, 182));
        tabbedPane.setVisible(true);

      JMenuBar menuBar = new JMenuBar();
      JMenu menuGK = new JMenu("GateKeeper");
      JMenu menuEdit = new JMenu("Edit");
      JMenu menuView = new JMenu("View");
      JMenu menuCmd = new JMenu("GK Commands");
      JMenu menuCal = new JMenu("Calculate");

      JMenu menuAbout = new JMenu("About");

      JMenu menuSave = new JMenu("Save as...");
      JMenu menuCmdEP = new JMenu("EPs...");
      JMenu menuCmdCall = new JMenu("Calls...");
      JMenu menuCmdCach = new JMenu("Cached Calls...");
      JMenu menuCmdDebug = new JMenu("Debug...");

      JMenuItem itmAbout = new JMenuItem("About");
      itmAbout.addActionListener(
            // anonymous inner class
            new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                    buttonAbout_actionPerformed(e);
                }
            }
        );

      JMenuItem itmGKMonitor = new JMenuItem("Monitor GK");
      itmGKMonitor.addActionListener(
            // anonymous inner class
            new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                    buttonMonitor_actionPerformed(e);
                }
            }
        );
      //itmGKMonitor.setActionCommand(MONITOR);

      JMenuItem itmGKConnect = new JMenuItem("Connect GK");
      itmGKConnect.setActionCommand(CONNECT);
      itmGKConnect.addActionListener(new ActionListener(){
              public void actionPerformed(ActionEvent e) {
                        GateKeeper theGK = (GateKeeper)tabbedPane.getSelectedComponent();
                        theGK.actionPerformed(e);
              }
          });


      JMenuItem itmGKDisc = new JMenuItem("Disconnect GK");
      itmGKDisc.setActionCommand(DISCONNECT);
      itmGKDisc.addActionListener(new ActionListener(){
              public void actionPerformed(ActionEvent e) {
                        GateKeeper theGK = (GateKeeper)tabbedPane.getSelectedComponent();
                        theGK.actionPerformed(e);
              }
          });

      JMenuItem itmGKExit = new JMenuItem("Exit");
      //itmGKExit.setActionCommand(EXIT);
      itmGKExit.addActionListener(
            // anonymous inner class
            new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                    buttonExit_actionPerformed(e);
                }
            }
        );

      itmViewBtn = new JMenuItem("Button List Mode");
      itmViewBtn.disable();
      itmViewBtn.setActionCommand(BUTTONLISTMODE);
      itmViewBtn.addActionListener(new ActionListener(){
              public void actionPerformed(ActionEvent e) {
                        GateKeeper theGK = (GateKeeper)tabbedPane.getSelectedComponent();
                        theGK.actionPerformed(e);
                        setBtnDis();

              }
          });

      itmViewTree = new JMenuItem("Tree List Mode");
      itmViewTree.setActionCommand(TREELISTMODE);
      itmViewTree.addActionListener(new ActionListener(){
              public void actionPerformed(ActionEvent e) {
                        GateKeeper theGK = (GateKeeper)tabbedPane.getSelectedComponent();
                        theGK.actionPerformed(e);
                        setTreeDis();
              }
          });

      JMenuItem itmEditClr = new JMenuItem("Clear log");
      itmEditClr.setActionCommand(CLEARBROWSER);
      itmEditClr.addActionListener(new ActionListener(){
              public void actionPerformed(ActionEvent e) {
                        GateKeeper theGK = (GateKeeper)tabbedPane.getSelectedComponent();
                        theGK.actionPerformed(e);
              }
      });

      JMenuItem itmEditRefresh = new JMenuItem("Refresh");
      itmEditRefresh.setActionCommand(REFRESHALL);
      itmEditRefresh.addActionListener(new ActionListener(){
              public void actionPerformed(ActionEvent e) {
                        GateKeeper theGK = (GateKeeper)tabbedPane.getSelectedComponent();
                        theGK.actionPerformed(e);
              }
      });

      JMenuItem itmGKSession = new JMenuItem("Close Session");
      itmGKSession.setActionCommand(CLOSESESSION);
      itmGKSession.addActionListener(new ActionListener(){
              public void actionPerformed(ActionEvent e) {
                        GateKeeper theGK = (GateKeeper)tabbedPane.getSelectedComponent();
                        theGK.actionPerformed(e);
              }
      });

      JMenuItem itmEditFindEP = new JMenuItem("Find an EP");
      itmEditFindEP.addActionListener(new ActionListener(){
          public void actionPerformed(ActionEvent e){
            FindEPDlg find = new FindEPDlg();
            find.setBackground(new Color(25, 69, 165));
            find.setLocation(300,300);
            find.show();
          }
      });
      JMenuItem itmEditSaveLog = new JMenuItem("Save GK Status Log");

      JMenuItem itmEditSaveCDR = new JMenuItem("Save Call Detail Record");

      JMenuItem itmCmdEPCur = new JMenuItem("Current EPs");
      itmCmdEPCur.addActionListener(
            // anonymous inner class
            new ActionListener()  {
                public void actionPerformed(ActionEvent e) {
                    currentEPBtn_actionPerformed(e);
                }
            }
        );

      JMenuItem itmCmdEPDtl = new JMenuItem("EPs Detail");
      itmCmdEPDtl.addActionListener(
            // anonymous inner class
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    currentEPVerboseBtn_actionPerformed(e);
                }
            }
        );

      JMenuItem itmCmdCallCur = new JMenuItem("Current Calls");
      itmCmdCallCur.addActionListener(
            // anonymous inner class
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    currentCallBtn_actionPerformed(e);
                }
            }
        );
      JMenuItem itmCmdCallDtl = new JMenuItem("Calls Detail");
      itmCmdCallDtl.addActionListener(
            // anonymous inner class
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    currentCallVerboseBtn_actionPerformed(e);
                }
            }
        );
      JMenuItem itmCmdCachCur = new JMenuItem("Current Cached Calls");
      itmCmdCachCur.addActionListener(
            // anonymous inner class
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    cachedEPBtn_actionPerformed(e);
                }
            }
        );
      JMenuItem itmCmdCachDtl = new JMenuItem("Cached Calls Detail");
      itmCmdCachDtl.addActionListener(
            // anonymous inner class
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    cachedEPVerboseBtn_actionPerformed(e);
                }
            }
        );
      JMenuItem itmCmdDebugFrm = new JMenuItem("Debug Frame");
      itmCmdDebugFrm.addActionListener(
            // anonymous inner class
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    gkCmdFrameDebugBtn_actionPerformed(e);
                }
            }
        );
      JMenuItem itmCmdDebugCns = new JMenuItem("Debug Console");
      itmCmdDebugCns.addActionListener(
            // anonymous inner class
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    gkCmdConsoleDebugBtn_actionPerformed(e);
                }
            }
        );

      JMenuItem itmCalUseTime = new JMenuItem("Total Use Time");
      itmCalUseTime.setActionCommand(TOTALUSETIME);
      itmCalUseTime.addActionListener(new ActionListener(){
              public void actionPerformed(ActionEvent e) {
                        GateKeeper theGK = (GateKeeper)tabbedPane.getSelectedComponent();
                        theGK.actionPerformed(e);
              }
      });

      JMenuItem itmCmdHelp = new JMenuItem("Command Help");
      itmCmdHelp.addActionListener(
            // anonymous inner class
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    gkCmdHelpBtn_actionPerformed(e);
                }
            }
        );

      menuBar.add(menuGK);
      menuBar.add(menuEdit);
      menuBar.add(menuView);
      menuBar.add(menuCmd);
      menuBar.add(menuCal);
      menuBar.add(menuAbout);
      menuGK.add(itmGKMonitor);
      menuGK.add(itmGKDisc);
      menuGK.add(itmGKConnect);
      menuGK.add(itmGKSession);
      menuGK.add(itmGKExit);
      menuEdit.add(itmEditClr);
      menuEdit.add(itmEditFindEP);
      menuEdit.add(menuSave);
      menuEdit.add(itmEditRefresh);
      menuSave.add(itmEditSaveLog);
      menuSave.add(itmEditSaveCDR);
      menuView.add(itmViewBtn);
      menuView.add(itmViewTree);
      menuCmd.add(menuCmdEP);
      menuCmdEP.add(itmCmdEPCur);
      menuCmdEP.add(itmCmdEPDtl);
      menuCmd.add(menuCmdCall);
      menuCmdCall.add(itmCmdCallCur);
      menuCmdCall.add(itmCmdCallDtl);
      menuCmd.add(menuCmdCach);
      menuCmdCach.add(itmCmdCachCur);
      menuCmdCach.add(itmCmdCachDtl);
      menuCmd.add(menuCmdDebug);
      menuCmdDebug.add(itmCmdDebugFrm);
      menuCmdDebug.add(itmCmdDebugCns);
      menuCmd.add(itmCmdHelp);
      menuCal.add(itmCalUseTime);
      menuAbout.add(itmAbout);

      contentPane.add(menuBar, BorderLayout.NORTH);
      contentPane.add(statusBar, BorderLayout.SOUTH);
      contentPane.add(tabbedPane, BorderLayout.CENTER);
    }
    public void setBtnDis(){
        itmViewBtn.setEnabled(false);
        itmViewTree.setEnabled(true);
      }

      public void setTreeDis(){
         itmViewBtn.setEnabled(true);
         itmViewTree.setEnabled(false);
      }

    /** Overridden so we can exit when window is closed */
    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            buttonExit_actionPerformed(null);
        }
    }
}
