package omnigkgui;

import java.io.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.net.*;
import java.awt.event.*;
import java.util.*;

import omnigkgui.gkcmd.GkCmdConstants;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huang, Hsiao-Mei
 * @version      1.0
 */

public class DebugConsoleFrame extends JDialog implements Runnable {
    // socket io to communicate with the gatekeeper
    private Socket socket;
    private InputStream istream;
    private OutputStream ostream;
    private String hostname;
    private int port;

    // text area for debugging message and input commands
    protected JPanel globalPane = new JPanel(new BorderLayout());
    protected JScrollPane scrollPane;
    protected JTextArea debugMessagePane;
    protected JTextArea commandLineInput;

    /** Construct the DebugFrame object */
    public DebugConsoleFrame(Frame parent, String title) {
        super(parent, title);

        // initialize the GUI components
        debugMessagePane = new JTextArea();
        debugMessagePane.setFont(new Font("Times New Roman", Font.BOLD, 16));
        debugMessagePane.setBackground(Color.black);
        debugMessagePane.setForeground(Color.green);
        debugMessagePane.setLineWrap(true);
        debugMessagePane.setEditable(false);
        scrollPane = new JScrollPane(debugMessagePane);

        commandLineInput = new JTextArea();
        commandLineInput.setFont(new Font("Helvetica", Font.PLAIN, 14));
        commandLineInput.setBackground(new Color(255, 217, 179));
        commandLineInput.setForeground(Color.red);
        commandLineInput.setLineWrap(true);
        commandLineInput.setEditable(true);
        commandLineInput.setRows(3);
        EtchedBorder border = new EtchedBorder(EtchedBorder.RAISED, Color.cyan, Color.blue);
        commandLineInput.setBorder(border);
        commandLineInput.addKeyListener(
            new java.awt.event.KeyAdapter() {
                public void keyPressed(KeyEvent e) {
                    commandLineInput_keyPressed(e);
                }
            }
        );

        // add components to the container panel
        globalPane.add(scrollPane, BorderLayout.CENTER);
        globalPane.add(commandLineInput, BorderLayout.SOUTH);

        this.getContentPane().add(globalPane);
        setVisible(true);
    }

    public void setGKIPAddress(String _hostname, int _port) {
        hostname = _hostname;
        port = _port;
    }

    public void stop() {
        try {
            // cleanup
            if (istream != null)
                istream.close();
            istream = null;
            if (ostream != null)
                ostream.close();
            ostream = null;
            if (socket != null)
                socket.close();
            socket = null;
        } catch (IOException e) {
            return;
        }
    }

    public void run() {
        if (istream != null) {
            return;
        }

        try {
            // open socket connection to GK
            socket = new Socket(hostname, port);

            ostream = socket.getOutputStream();
            istream = socket.getInputStream();
            BufferedReader in = new BufferedReader(new InputStreamReader(istream));

            debugMessagePane.append("\nDebug Commands Usages Explaination : \n");
            ostream.write("Debug\n".getBytes());
            // list the available sections
            debugMessagePane.append("\n Current Available Sections is --> \n\n");
            for (int i = 0; i < GkCmdConstants.sections.length; i++) {
                debugMessagePane.append("\t[ " + GkCmdConstants.sections[i++] + " ]");
                if (i < GkCmdConstants.sections.length)
                    debugMessagePane.append("    [ " + GkCmdConstants.sections[i] + " ]\n");
                else
                    debugMessagePane.append("\n\n");
            }

            String inputLine;
            while ((inputLine = in.readLine()) != null) {
                if (inputLine.startsWith(";"))
                    debugMessagePane.append("***** End Debug Session *****\n");
                if ((!inputLine.endsWith(";")) && (!inputLine.startsWith("CDR")))
                    debugMessagePane.append(inputLine + "\n");
            }
            stop();
        } catch (Exception e) {
            return;
        }
    }

/*  public DebugFrame() {
        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private void jbInit() throws Exception {
        jTextArea1.setText("jTextArea1");
        jTextArea1.addKeyListener(
            new java.awt.event.KeyAdapter() {
                public void keyPressed(KeyEvent e) {
                    jTextArea1_keyPressed(e);
                }
            }
        );
        this.getContentPane().add(jTextArea1, BorderLayout.CENTER);
    }
*/

    // event handler for Command Line Input pane when "Enter" is pressed
    void commandLineInput_keyPressed(KeyEvent e) {
        try {
            if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                String str = commandLineInput.getText();
                StringTokenizer tokens = new StringTokenizer(str, " ");
                if ((tokens.nextToken()).equals("Debug") && tokens.hasMoreTokens()) {
                    debugMessagePane.append("\n***** Start Debug Session *****\n");
                    debugMessagePane.append(str + "\n\n");
                    ostream.write((str + "\n").getBytes());
                } else if (str.startsWith("Debug") || str.equals("Help")) {
                    // clear the text pane
                    debugMessagePane.setText(null);
                    /*
                    debugMessagePane.append("\n\nDebug Commands Usages Explaination : \n");
                    ostream.write((str + "\n").getBytes());
                    */
                    showUsage();
                } else if (str.equals("clear")) {
                    // just clear the text pane
                    debugMessagePane.setText(null);
                } else {
                    // clear the text pane
                    debugMessagePane.setText(null);

                    debugMessagePane.append("\nInvalid Command Formats !!\nSee below\n");
                    showUsage();
                }
                commandLineInput.setText(null);
                e.setKeyCode(KeyEvent.CHAR_UNDEFINED);
            }
        } catch (IOException ioe) {
            debugMessagePane.append("Error : " + ioe + "\n");
            return;
        }
    }

    // method to print the usage
    private void showUsage() {
        try {
            debugMessagePane.append("\nDebug Commands Usages Explaination : \n");
            ostream.write(("Debug\n").getBytes());

            // list the available sections
            debugMessagePane.append("\n Current Available Sections is --> \n\n");
            for (int i = 0; i < GkCmdConstants.sections.length; i++) {
                debugMessagePane.append("\t[ " + GkCmdConstants.sections[i++] + " ]");
                if (i < GkCmdConstants.sections.length)
                    debugMessagePane.append("    [ " + GkCmdConstants.sections[i] + " ]\n");
                else
                    debugMessagePane.append("\n\n");
            }
        } catch (IOException ioe) {
            debugMessagePane.append("Error : " + ioe + "\n");
            return;
        }
    }

}