package omnigkgui;

import javax.swing.*;
import java.awt.*;
import java.util.*;
import javax.swing.tree.*;
import java.io.*;

import omnigkgui.util.*;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huang, Hsiao-Mei
 * @version      1.0
 */

public class EndPoint
{
    public final static int AVAILABLE = 0;
    public final static int TALKING = 1;

    public String epID;
    public String epIP;
    public String epAlias;
    public String epType;
    public int state;
    public JButton button;
    public DefaultMutableTreeNode treeNode;

    private Properties callTBL;
    private int totalInCallNum, currentInCallNum;
    private int totalOutCallNum, currentOutCallNum;
    private int activeCallNum;

    /** Construct the EndPoint object */
    public EndPoint(String _epid, String _epip, String _epalias, String _eptype, JButton _button) {
        epID = _epid;
        epIP = _epip;
        epAlias = _epalias;
        epType = _eptype;
        state = AVAILABLE;
        button = _button;

        callTBL = new Properties();
        totalInCallNum = currentInCallNum = 0;
        totalOutCallNum = currentOutCallNum = 0;
        activeCallNum = 0;
    }

    /** public interfaces */
    public void setStateAvailable() {
        if (epType.equals("gateway")) {
            button.setBackground(Color.pink);
            button.setForeground(Color.white);
        } else if (epType.equals("terminal"))
            button.setBackground(Color.green);
        else
            button.setBackground(Color.yellow);
    }

    public void insertCallRecord(String _epid, String _callRF) {
        if (callTBL.get(_callRF) == null) {
            // add new record into Call Table
            callTBL.put(_callRF, _epid);
        } else {
            // duplicate ACF, just do nothing
        }
    }

    public void removeCallRecord(String _callRF) {
        callTBL.remove(_callRF);
    }

    public boolean findCallRecord(String _callRF) {
        return ((callTBL.get(_callRF) == null) ? false : true);
    }

    public boolean isCallRecordEmpty() {
        return callTBL.isEmpty();
    }

    public int getCurrentInCallNum() {
        return currentInCallNum;
    }

    public void addCurrentInCall() {
        currentInCallNum++;
    }

    public void delCurrentInCall() {
        if (currentInCallNum > 0)
            currentInCallNum--;
    }

    public int getTotalInCallNum() {
        return totalInCallNum;
    }

    public void addTotalInCall() {
        totalInCallNum++;
    }

    public void resetTotalInCallNum() {
        totalInCallNum = 0;
    }

    public int getCurrentOutCallNum() {
        return currentOutCallNum;
    }

    public void addCurrentOutCall() {
        currentOutCallNum++;
    }

    public void delCurrentOutCall() {
        if (currentOutCallNum > 0)
            currentOutCallNum--;
    }

    public int getTotalOutCallNum() {
        return totalOutCallNum;
    }

    public void addTotalOutCall() {
        totalOutCallNum++;
    }

    public void resetTotalOutCallNum() {
        totalOutCallNum = 0;
    }

    public int getActiveCallNum() {
        return activeCallNum;
    }

    public void addActiveCall() {
        activeCallNum++;
    }

    public void delActiveCall() {
        if (activeCallNum > 0)
            activeCallNum--;
    }

    public void resetActiveCallNum() {
        activeCallNum = 0;
    }

    /** private or protected interfaces */
    protected boolean isEqualEPByAliasList(String srcInfo) {
        boolean isMatched = false;
        ParseAlias srcAlias = new ParseAlias(srcInfo);
        Vector tmpVec = null;

        if ((tmpVec = srcAlias.getDialedDigitsValues()) != null) {
            int size = tmpVec.size();
            for (int i = 0; i < size; i++) {
                if ((epAlias.indexOf(tmpVec.get(i).toString())) == -1)
                    return false;
            }
            isMatched = true;
        }

        if ((tmpVec = srcAlias.getH323IDValues()) != null) {
            int size = tmpVec.size();
            for (int i = 0; i < size; i++) {
                if ((epAlias.indexOf(tmpVec.get(i).toString())) == -1)
                    return false;
            }
            isMatched = true;
        }

        return isMatched;
    }


    protected String getEPDetailInfo() {
        // set the detail endpoint information
        //display when the mouse is on an EP
        ParseAlias alias = new ParseAlias(epAlias);
        String detailInfo = "";
        detailInfo = "<html><font size=5 color=#8080C0><center>" +
                            "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                            " Endpoint Information " +
                            "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                            "</center></font><hr>";
        detailInfo += "<center><table border=0 width=100% align=center>";

        int ipIndex = 0;
        String detail_IP = "";
        if ((ipIndex = epIP.indexOf(":")) != -1)
            detailInfo +=
                        "<tr>" +
                                "<td width=40% align=left>" +
                                        "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                        "<font size=4>" +
                                                "EP-IP :" +
                                        "</font>" +
                                "</td>" +
                                "<td width=60% align=left>" +
                                        "<font size=4 color=#FE821D><b><i>" +
                                                epIP.substring(0, ipIndex) + "&nbsp;&nbsp;" +
                                        "</i></b></font>" +
                                        "<font size=3 color=#0000FF><b><i>" +
                                                epIP.substring(ipIndex + 1) +
                                        "</i></b></font>" +
                                "</td>" +
                        "</tr>";
        else
            detailInfo +=
                        "<tr>" +
                                "<td width=40% align=left>" +
                                        "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                        "<font size=4>" +
                                                "EP-IP :" +
                                        "</font>" +
                                "</td>" +
                                "<td width=60% align=left>" +
                                        "<font size=4 color=#FE821D><b><i>" +
                                                epIP +
                                        "</i></b></font>" +
                                "</td>" +
                        "</tr>";

        detailInfo +=
                    "<tr>" +
                            "<td width=40% align=left>" +
                                    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                    "<font size=4>" +
                                            "EP-Type :" +
                                    "</font>" +
                            "</td>" +
                            "<td width=60% align=left>" +
                                    "<font size=4 color=#FE821D><b><i>" +
                                            epType.replace(',', '&') +
                                    "</i></b></font>" +
                            "</td>" +
                    "</tr>";

        detailInfo +=
                    "<tr>" +
                            "<td width=40% align=left>" +
                                    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                    "<font size=4>" +
                                            "EP-ID :" +
                                    "</font>" +
                            "</td>" +
                            "<td width=60% align=left>" +
                                    "<font size=4 color=#FF0000><b><i>" +
                                            epID +
                                    "</i></b></font>" +
                            "</td>" +
                    "</tr>";

        detailInfo +=
                    "<tr>" +
                            "<td width=40% align=left>" +
                                    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                    "<font size=4>" +
                                            "EP-Alias :" +
                                    "</font>" +
                            "</td>" +
                            "<td width=60% align=left></td>" +
                    "</tr>";
        // print the "DialedDigits" and "H323ID" format H323 alias
        Vector tmpVec = null;
        if ((tmpVec = alias.getDialedDigitsValues()) != null) {
            detailInfo +=
                        "<tr>" +
                                "<td width=40% align=left></td>" +
                                "<td width=60% align=left>" +
                                        "<font size=3><b>" +
                                                "[DialedDigits]" +
                                        "</b></font>" +
                                "</td>" +
                        "</tr>";
            int size = tmpVec.size();
            for (int i = 0; i < size; i++) {
                detailInfo +=
                            "<tr>" +
                                    "<td width=40% align=left></td>" +
                                    "<td width=60% align=left>" +
                                            "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                            "<font size=3 color=#FF00FF>" +
                                                    tmpVec.get(i).toString() +
                                            "</font>" +
                                    "</td>" +
                            "</tr>";
            }
        }
        tmpVec = null;
        if ((tmpVec = alias.getH323IDValues()) != null) {
            detailInfo +=
                        "<tr>" +
                                "<td width=40% align=left></td>" +
                                "<td width=60% align=left>" +
                                        "<font size=3><b>" +
                                                "[H323ID]" +
                                        "</b></font>" +
                                "</td>" +
                        "</tr>";
            int size = tmpVec.size();
            for (int i = 0; i < size; i++) {
                detailInfo +=
                            "<tr>" +
                                    "<td width=40% align=left></td>" +
                                    "<td width=60% align=left>" +
                                            "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                            "<font size=3 color=#FF00FF>" +
                                                    tmpVec.get(i).toString() +
                                            "</font>" +
                                    "</td>" +
                            "</tr>";
            }
        }
        // print the in-Call and out-Call and active-Call number
        detailInfo +=
                    "<tr>" +
                            "<td width=40% align=left>" +
                                    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                    "<font size=3><b>" +
                                            "CIN :" +
                                    "</b></font>" +
                            "</td>" +
                            "<td width=60% align=left>" +
                                    "<font size=4 color=#FF0000><b><i>" +
                                            getCurrentInCallNum() +
                                    "</i></b></font>" +
                            "</td>" +
                    "</tr>";

        detailInfo +=
                    "<tr>" +
                            "<td width=40% align=left>" +
                                    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                    "<font size=3><b>" +
                                            "TIN :" +
                                    "</b></font>" +
                            "</td>" +
                            "<td width=60% align=left>" +
                                    "<font size=4 color=#FF0000><b><i>" +
                                            getTotalInCallNum() +
                                    "</i></b></font>" +
                            "</td>" +
                    "</tr>";

        detailInfo +=
                    "<tr>" +
                            "<td width=40% align=left>" +
                                    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                    "<font size=3><b>" +
                                            "CON :" +
                                    "</b></font>" +
                            "</td>" +
                            "<td width=60% align=left>" +
                                    "<font size=4 color=#FF0000><b><i>" +
                                            getCurrentOutCallNum() +
                                    "</i></b></font>" +
                            "</td>" +
                    "</tr>";

        detailInfo +=
                    "<tr>" +
                            "<td width=40% align=left>" +
                                    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                    "<font size=3><b>" +
                                            "TON :" +
                                    "</b></font>" +
                            "</td>" +
                            "<td width=60% align=left>" +
                                    "<font size=4 color=#FF0000><b><i>" +
                                            getTotalOutCallNum() +
                                    "</i></b></font>" +
                            "</td>" +
                    "</tr>";

        detailInfo +=
                    "<tr>" +
                            "<td width=40% align=left>" +
                                    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" +
                                    "<font size=3><b>" +
                                            "Active Call :" +
                                    "</b></font>" +
                            "</td>" +
                            "<td width=60% align=left>" +
                                    "<font size=4 color=#FF0000><b><i>" +
                                            getActiveCallNum() +
                                    "</i></b></font>" +
                            "</td>" +
                    "</tr>";

        detailInfo += "</table></center><br></html>";
        return detailInfo;
    }
}
