package omnigkgui;

import javax.swing.*;
import java.net.*;
import java.io.*;
import java.sql.*;
import java.util.*;

import omnigkgui.util.*;
import omnigkgui.gkcmd.*;
import omnigkgui.*;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huang, Hsiao-Mei
 * @version      1.0
 */

public class GkClient implements Runnable {
    private GateKeeper gk;
    private String host;
    private int port;
    private Vector timeIDVec = new Vector();;

    // socket io to communicate with the gatekeeper
    private Socket socket;
    private InputStream istream;
    private OutputStream ostream;

    public GkClient() {

    }

    public void init(GateKeeper _gk, String _host, int _port) {
        gk = _gk;
        host = _host;
        port = _port;
    }

    public void stop() {
        try {
            // cleanup
            if (istream != null)
                istream.close();
            istream = null;
            if (ostream != null)
                ostream.close();
            ostream = null;
            if (socket != null)
                socket.close();
            socket = null;
        } catch (IOException e) {
            return;
        }
    }

    public void run() {
        if (istream != null) {
            return;
        }

        try {
            gk.appendLog("Connecting to gatekeeper at " + host + ":" + port + "\n");
            socket = new Socket(host, port);
        } catch (Exception e) {
            gk.appendLog("Error: " + e + "\n");
            return;
        }

        try {
            ostream = socket.getOutputStream();
            istream = socket.getInputStream();
            BufferedReader in = new BufferedReader(new InputStreamReader(istream));

            ostream.write("PrintAllRegistrations\n".getBytes());
            ostream.write("PrintCurrentCalls\n".getBytes());

            String inputLine;
            String timeID;
            String useTime;
            String callerEndpointID;
            while ((inputLine = in.readLine()) != null) {
                // append the event to log
                gk.appendLog(inputLine + "\n");

                // update endpoint buttons
                if (inputLine.startsWith("RCF")) {
                    String epid = extractField(inputLine, 4);
                    String alias = extractField(inputLine, 2);
	            String epip = extractField(inputLine, 1);
	            String eptype = extractField(inputLine, 3);
                    // add GUI component
                    gk.addEndpoint(epid, alias, eptype, epip);
                }
                if (inputLine.startsWith("UCF") || inputLine.startsWith("URQ")) {
                    String epid = extractField(inputLine, 2);
//	            String epip = extractField(inputLine, 1);
                    // add GUI component
                    gk.removeEndpoint(epid);
	        }
                if (inputLine.startsWith("ACF")) {
                    String epid = extractField(inputLine, 2);
                    String callRF = extractField(inputLine, 3);
                    String srcInfo = extractField(inputLine, 5);
                    // add GUI component
                    gk.setEndpointState(epid, EndPoint.TALKING);
                    gk.processNewCall(epid, callRF, srcInfo);
                }
                if (inputLine.startsWith("DCF")) {
                    String epid = extractField(inputLine, 2);
                    String callRF = extractField(inputLine, 3);
                    // add GUI component
                    gk.dropExitCall(epid, callRF);
                    gk.setEndpointState(epid, EndPoint.AVAILABLE);
                }

                if (inputLine.startsWith("CDR")) {
                    String callNumber = extractField(inputLine, 1);
                  //String callID = extractField(inputLine, 2);
                    String startTime = extractField(inputLine, 4);
                    String endTime = extractField(inputLine, 5);
                    useTime = extractField(inputLine, 3);
                    String account = null;
                    String dialCode = extractField(inputLine, 10);
                    String callerIPPort = extractField(inputLine, 6);
                    callerEndpointID = extractField(inputLine, 7);
                    String calleeIPPort = extractField(inputLine, 8);
                    String calleeEndpointID = extractField(inputLine, 9);

                    // decrease one from active call number
                    gk.dropExitCall(callerEndpointID, "CDRSRC");
                    gk.dropExitCall(calleeEndpointID, "CDRDEST");

                    if (callerIPPort.equals(" ") && callerEndpointID.equals(" ")) {
                        // call came from neighbor
                        callerIPPort = "ByNeighborGK";
                        callerEndpointID = "ByNeighborGK";
                        account = "Neighbor IPN";
                    }
                    // format the datetime object
                    if (startTime.equals("unconnected")) {
			if (endTime.equals(" ")) {
                            startTime = "0000-00-00 00:00:00";
                            endTime = "0000-00-00 00:00:00";
                        } else {
                            ParseDate spDateStr = new ParseDate(endTime);
                            startTime = endTime = spDateStr.toMySQLDateTimeString();
                        }
                        useTime = "0";
                    } else {
                        ParseDate spDateStr = new ParseDate(startTime);
                        startTime = spDateStr.toMySQLDateTimeString();
                        ParseDate epDateStr = new ParseDate(endTime);
                        endTime = epDateStr.toMySQLDateTimeString();
                    }

                    // find the account from epList's epAlias
                    String h323Alias = null;
                    for (int i = 0; i < gk.epList.size(); i++) {
                        EndPoint ep = (EndPoint) gk.epList.elementAt(i);
                        if (callerEndpointID.equals(ep.epID)) {
                            h323Alias = ep.epAlias;
                            break;
                        }
                    }

                    if (h323Alias != null) {
                        ParseAlias tmpPA = new ParseAlias(h323Alias);
                        account = tmpPA.getDialedDigits();
                        if (account == null)
                            account = tmpPA.getH323ID();
                        else if (account == null)
                            account = "UnKnown IPN";
                    }
                    // strip the ":dialedDidits" string of dialCode
                    if (!dialCode.equals("unknown"))
                        dialCode = (new ParseAlias(dialCode)).getDialedDigits();

                    // add cdr to table
                    Object[] newCDR = new Object[] { callNumber, startTime, endTime,
                                            useTime, account, dialCode, callerIPPort, calleeIPPort };
                    gk.cdrTableModel.addRow(newCDR);
                    // don't update the UI, it may produce the Event-Dispatching runtime exception
                    //gk.cdrTable.updateUI();

                    //get the information of useTime and callerEndpointID
                    timeID = useTime + "+" + account;
                    timeIDVec.add(timeID);
                    timeID = null;
                }
            }
            stop();
        } catch (IOException e) {
            return;
        }
        // app.appendLog("Client thread stopped\n");
    }



     public String totalUseTimeOfEP(String _callerID){
        int intUseTime = 0;
        int totalUseTime = 0;
        int plusIndex = 0;
        String plus = "+";
        String strTime = null;
        String strID = null;
      if(timeIDVec.size() == 0) return null;
        Enumeration enumTemp = timeIDVec.elements();
        while(enumTemp.hasMoreElements()){
          String temp = (String)enumTemp.nextElement();
          //find the index of "+" in the String temp
          plusIndex = temp.indexOf(plus);
            //get the EPID
            strID = temp.substring(plusIndex+1, temp.length());
            if(strID.equals(_callerID)){
              //get the useTime
              strTime = temp.substring(0, plusIndex);
              intUseTime = Integer.parseInt(strTime);
              totalUseTime = totalUseTime + intUseTime;
            }
        }
        return String.valueOf(totalUseTime);
    }

    public void refresh() {
        if (ostream != null) {
            try {
                gk.flushTextPane();
                gk.appendLog("Connecting to gatekeeper at " + host + ":" + port + "\n");
                ostream.write("PrintAllRegistrations\n".getBytes());
                ostream.write("PrintCurrentCalls\n".getBytes());
            } catch (Exception e) {
                gk.appendLog("Exception: " + e + "\n");
            }
        }
    }

    public void refreshWithoutCommands() {
        if (ostream != null) {
            try {
                gk.flushTextPane();
                gk.appendLog("Connecting to gatekeeper at " + host + ":" + port + "\n");
            } catch (Exception e) {
                gk.appendLog("Exception: " + e + "\n");
            }
        }
    }

    // extract a field (counting from one) from a pipe-seperated string,
    // format --> ("header |field 1| field 2|field 3")
    protected String extractField(String _msg, int _which) {
        String token = "";
        int start = 0, end = 0;

        if (_msg.endsWith(";"))
            _msg = _msg.substring(0, _msg.length() - 1);
        _msg += "|";

        for (int i = 0; i < _which; i++) {
            if (start < _msg.length())
                start = _msg.indexOf("|", start) + 1;
            else
                break;
        }
        end = _msg.indexOf("|", start);

        try {
            token = _msg.substring(start, end);
        } finally {
            return token;
        }
    }

    // transfer GK command to GK
    protected void transferCommand(String comment, String cmd) {
        try {
            gk.appendLog(comment);
            ostream.write(cmd.getBytes());
        } catch (IOException e) {
            gk.appendLog("Error: " + e + "\n");
            return;
        }
    }

 public void removeEPOnGK(Vector epVec){

      //remove this ep from gk by alias

      String epid = (String)epVec.get(0);
      String epipPort = (String)epVec.get(2);
      String mark = ":";
      int markInx = epipPort.indexOf(mark);
      String epip = epipPort.substring(0, markInx);

      try{
      ostream.write(("unregisterip " + epip + "\n").getBytes());
      //alert(_epid + "is removed from GK!");
      }catch(IOException e){
        e.printStackTrace();
      }
      gk.removeEndpoint(epid);
     }
 }
