package omnigkgui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.*;
import javax.print.*;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huang, Hsiao-Mei
 * @version      1.0
 */

public class LogFileHandler extends JPopupMenu implements MouseListener, ActionListener {
    protected final static String ITEM_ONE = "Save";
    protected final static String ITEM_TWO = "Save as ...";
    protected final static String ITEM_THREE = "Print";

    private GateKeeper theGK;
    private Component selectedComponent;
    private JFileChooser chooser;

    public LogFileHandler(GateKeeper gk) {
        super("LogFile");
        theGK = gk;
        this.add(makeMenuItem(ITEM_ONE));
        this.add(makeMenuItem(ITEM_TWO));
        this.add(makeMenuItem(ITEM_THREE));
        setVisible(false);
    }

    private JMenuItem makeMenuItem(String label) {
        JMenuItem item = new JMenuItem(label);
        item.setBackground(SystemColor.info);
        item.setForeground(SystemColor.infoText);
        item.addActionListener(this);
        return item;
    }

    public void actionPerformed(ActionEvent e) {
        String identifier = e.getActionCommand();

        if (identifier.equals(ITEM_TWO)) {
            // do wanted actions here
            chooser = new JFileChooser(".");
            if (selectedComponent.getClass().getName().equals("javax.swing.JTable")) {
                int returnVal = chooser.showSaveDialog(this);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    File logFile = chooser.getSelectedFile();
                    try {
                        JTable tmpTable = (JTable)selectedComponent;
                        int rowCount = tmpTable.getRowCount();
                        int columnCount = tmpTable.getColumnCount();
                        FileWriter logWriter = new FileWriter(logFile);

                        // write the column name value
                        logWriter.write(tmpTable.getColumnName(0));
                        for (int i = 1; i < columnCount; i++) {
                            logWriter.write(",");
                            logWriter.write(tmpTable.getColumnName(i));
                        }
                        logWriter.write("\r\n");
                        // write every row content
                        for (int i = 0; i < rowCount; i++) {
                            logWriter.write(tmpTable.getValueAt(i, 0).toString());
                            for (int j = 1; j < columnCount; j++) {
                                logWriter.write(",");
                                logWriter.write(tmpTable.getValueAt(i, j).toString());
                            }
                            logWriter.write("\r\n");
                        }

                        logWriter.close();
                        theGK.removeCDRRecord(0, rowCount);
                    } catch (IOException ioe) {
                        ioe.printStackTrace();
                    }
                }
            } else if (selectedComponent.getClass().getName().equals("javax.swing.JTextPane")) {
                int returnVal = chooser.showSaveDialog(this);

                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    File logFile = chooser.getSelectedFile();
                    try {
                        JTextPane tmpTextPane = (JTextPane)selectedComponent;
                        FileWriter logWriter = new FileWriter(logFile);

                        // write the log to file
                        logWriter.write(tmpTextPane.getText());

                        logWriter.close();
                        theGK.flushTextPane();
                        theGK.appendLog("Connecting to gatekeeper at " + theGK.getHostName() + ":" + theGK.getPort() + "\n");
                    } catch (IOException ioe) {
                        ioe.printStackTrace();
                    }
                }
            }
        } else if (identifier.equals(ITEM_ONE)) {
            JOptionPane.showMessageDialog(this,
                                    "This is a test show message\n" +
                                    "It really does nothing\n" +
                                    "Please see the next \"Save as ...\" command",
                                    "Save Dialog Box Message",
                                    JOptionPane.INFORMATION_MESSAGE);
        } else if (identifier.equals(ITEM_THREE)){
          //DocFlavor doc = new DocFlavor("text/html;charset=utf-16", "java.io.InputStream");
          //DocFlavor flavor = DocFlavor.INPUT_STREAM.POSTSCRIPT;

          /**if(PrintServiceLookup.getPrintServices()){
          PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
          aset.add(MediaSizeName.ISO_A4);
          PrintService[] service =
	  PrintServiceLookup.lookupPrintServices(flavor, aset);
          } else {
            JOptionPane.showMessageDialog(this, "You don't have printer.");
          }

          PrintService printer = PrintServiceLookup.lookupDefaultPrintService();
          if(printer){
            DocPrintJob printJob = new DocPrintJob();
            PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
            printJob.print(flavor, aset);
            JTextPane tmpTextPane = (JTextPane)selectedComponent;
          } else {
            JOptionPane.showMessageDialog(this, "You don't have printer.");
          }*/
        }
    }

    public void mouseClicked(MouseEvent e) {
        // right button is pressed
        if (SwingUtilities.isRightMouseButton(e)) {
            selectedComponent = e.getComponent();
            this.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    public void mouseEntered(MouseEvent e) {
        // right button is pressed
        if (SwingUtilities.isRightMouseButton(e)) {
            selectedComponent = e.getComponent();
            this.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    public void mousePressed(MouseEvent e) {
        // right button is pressed
        if (SwingUtilities.isRightMouseButton(e)) {
            selectedComponent = e.getComponent();
            this.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    public void mouseReleased(MouseEvent e) {
        // do nothing
    }

    public void mouseExited(MouseEvent e) {
        // do nothing
    }
}