package omnigkgui.gkcmd;

import java.io.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huagn, Hisao-Mei
 * @version      1.0
 */

public class GkCmdConfigMsg extends ShownMsgFrame implements Runnable {
    // iostream for communicate with gatekeeper
    private PipedReader istream = null;
    private OutputStream ostream = null;
    private BufferedReader inBuf = null;

    // main message pane
    private JTabbedPane msgTabbedPane = new JTabbedPane();

    public GkCmdConfigMsg(PipedWriter in, OutputStream out, GkCommand commander) {
        super("GK Commands : Gatekeeper Configuration" + "  <" + commander.getHost() + ">");
        super.setCommander(commander);

        try {
            // initialize the iostream
            istream = new PipedReader(in);
            ostream = out;

            if (istream != null) {
                inBuf = new BufferedReader(istream);
            }
            // call the main initialization function
            jbInit();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        // set the font and parameters for "msgTabbedPane"
        msgTabbedPane.setFont(new Font("Currier New", Font.BOLD, 12));
        // msgTabbedPane.setBackground(SystemColor.info);
        msgTabbedPane.setForeground(Color.blue);

        // add the "msgTabbedPane" into the frame
        super.getContentPane().add(msgTabbedPane, null);
        this.setVisible(true);
    }

    public void run() {
        // add all recognized gatekeeper configuration sections
        for (int i = 0; i < GkCmdConstants.sections.length; i++) {
            // initialize all the GUI components
            JScrollPane scrollPane = new JScrollPane();
            JPanel secPane = new JPanel();
            secPane.setLayout(new BorderLayout(10, 20));
            secPane.setBackground(SystemColor.info);

            TitledBorder border = new TitledBorder(GkCmdConstants.sections[i]);
            border.setTitleFont(new Font("Helvetica", Font.ITALIC, 16));
            border.setTitleColor(Color.red);

            JTextArea usage = new JTextArea();
            usage.setFont(new Font("Helvetica", Font.BOLD, 14));
            usage.setBorder(new LineBorder(Color.pink, 5, true));

            JPanel settingPane = new JPanel();
            settingPane.setLayout(new GridLayout(0, 3, 4, 2));
            settingPane.setBorder(new LineBorder(Color.darkGray, 3, true));

            JPanel actionPane = new JPanel();
            actionPane.setBorder(new LineBorder(Color.green, 2, true));

            // setting the usage mesaage panel--(JTextArea usage)
            if (i <= 2) {
                usage.setText(GkCmdConstants.configUsages[i]);
            } else {
                usage.setText(GkCmdConstants.configUsages[0]);
            }

            // setting the GK params panel--(JPanel settingPane)
            boolean hasParams = false;
            try {
                // get params received from gatekeeper
//              super.getCommander().setGkCmdFlag(GkCmdIDs.GKCmdDebug);
                ostream.write(("Debug cfg " + GkCmdConstants.sections[i] + "\n").getBytes());

                // add the column headers
                JLabel columnKey = new JLabel("Key", SwingConstants.CENTER);
                JLabel columnValue = new JLabel("Value", SwingConstants.CENTER);
                JLabel columnAction = new JLabel("Action", SwingConstants.CENTER);
                columnKey.setBackground(Color.lightGray);
                columnValue.setBackground(Color.lightGray);
                columnAction.setBackground(Color.lightGray);
                settingPane.add(columnKey);
                settingPane.add(columnValue);
                settingPane.add(columnAction);

                // add parameters received from gatekeeper
                String inputLine;
                while ((inputLine = inBuf.readLine()) != null) {
                    if (inputLine.equals("EOF_STREAM")) {
                        // break the while loop
                        break;
                    } else {
                        if (!inputLine.startsWith("Section")) {
                            // set the tag for identifing if this section has at least one key-value pair
                            hasParams = true;

                            String keyStr = inputLine.substring(0, inputLine.indexOf("="));
                            int option_mode = optionStatus(keyStr, i);

                            JLabel key = new JLabel(keyStr, SwingConstants.LEFT);
                            JTextField value = new JTextField(inputLine.substring(inputLine.indexOf("=") + 1, inputLine.length()));
                            JPanel actionPanel = new JPanel();
                            JButton actionModifyBtn = null;
                            JButton actionDeleteBtn = null;

                            switch (option_mode) {
                                case 0 :
                                        key.setBorder(new LineBorder(Color.cyan, 1));
                                        key.setFont(new Font("Helvetica", Font.BOLD, 14));
                                        key.setForeground(Color.red);

                                        value.setBorder(new LineBorder(Color.cyan, 1));
                                        value.setFont(new Font("Currier New", Font.BOLD, 12));

                                        actionPanel.setBorder(new LineBorder(Color.cyan, 1));
                                        actionModifyBtn = new JButton(" Modify ");
                                        actionDeleteBtn = new JButton(" Delete ");
                                        actionPanel.add(actionModifyBtn);
                                        actionPanel.add(actionDeleteBtn);
                                        break;
                                case 1 :
                                        key.setBorder(new LineBorder(Color.orange, 1));
                                        key.setFont(new Font("Helvetica", Font.BOLD, 14));
                                        key.setForeground(Color.blue);

                                        value.setBorder(new LineBorder(Color.orange, 1));
                                        value.setFont(new Font("Currier New", Font.BOLD, 12));

                                        actionPanel.setBorder(new LineBorder(Color.orange, 1));
                                        actionModifyBtn = new JButton(" Modify ");
                                        actionPanel.add(actionModifyBtn);
                                        break;
                                case 2 :
                                        key.setBorder(new LineBorder(Color.darkGray, 1));
                                        key.setFont(new Font("Helvetica", Font.ITALIC, 14));
                                        key.setForeground(Color.black);

                                        value.setBorder(new LineBorder(Color.darkGray, 1));
                                        value.setFont(new Font("Helvetica", Font.ITALIC, 12));
                                        value.setBackground(key.getBackground());
                                        value.setEditable(false);
                                        break;
                                default :
                                        key.setBorder(new LineBorder(Color.red, 1));
                                        key.setBackground(Color.red);
                                        key.setOpaque(true);
                                        key.setFont(new Font("Helvetica", Font.PLAIN, 12));
                                        key.setForeground(Color.white);

                                        value.setBorder(new LineBorder(Color.red, 1));
                                        value.setBackground(Color.red);
                                        value.setFont(new Font("Currier New", Font.PLAIN, 12));
                                        value.setForeground(Color.white);

                                        actionPanel.setBackground(Color.red);
                                        actionPanel.setLayout(new BorderLayout());
                                        JLabel error = new JLabel(" Maybe the Error Option ");
                                        error.setFont(new Font("Currier New", Font.PLAIN, 12));
                                        error.setForeground(Color.white);
                                        actionPanel.add(error, BorderLayout.CENTER);
                            }
                            // add components into settingPane
                            settingPane.add(key);
                            settingPane.add(value);
                            settingPane.add(actionPanel);
                        }
                    }
                }
            } catch (IOException ioe) {
                // do nothing, just ignore
            }

            // setting the actions panel--(JPanel actionPane)
            JButton addKey = new JButton(" Add key ");
            JButton removeKey = new JButton(" Remove key ");
            actionPane.add(addKey);
            actionPane.add(removeKey);

            // combine all components
            secPane.setBorder(border);
            secPane.add(usage, BorderLayout.NORTH);
            if (hasParams != false) {
                secPane.add(settingPane, BorderLayout.CENTER);
                secPane.add(actionPane, BorderLayout.SOUTH);
            } else {
                secPane.add(actionPane, BorderLayout.CENTER);
            }
            scrollPane.setViewportView(secPane);

            // only show the sections which have at least one key-value pair
            if ((!GkCmdConstants.shownAtLeastOneValueSection) ||
                ((GkCmdConstants.shownAtLeastOneValueSection) && (hasParams != false)))
                msgTabbedPane.addTab(GkCmdConstants.sections[i], null, scrollPane);
        }
        super.getCommander().setGkCmdFlag(0);
    }

    /** private interfaces */
    private int optionStatus(String key, int whichSection) {
        // check if the section is belong to "dynamic_option"
        if (GkCmdConstants.section_options[whichSection][0].equals("dynamic_option"))
            return 0;

        for (int i = 0; i < GkCmdConstants.section_options[whichSection].length; i++) {
            if (GkCmdConstants.section_options[whichSection][i].startsWith(key))
                // 1 : can be modified by user, 2 : can't be modified
                return Integer.valueOf(GkCmdConstants.section_options[whichSection][i].substring(GkCmdConstants.section_options[whichSection][i].indexOf(":") + 1)).intValue();
        }

        // not find in the list, just return -1
        return -1;
    }
}