package omnigkgui.gkcmd;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huagn, Hisao-Mei
 * @version      1.0
 */

public class GkCmdConstants {
    /** global public gatekeeper commands constants */

    // constants defination for MsgDlg Size
    public static final int SHOWNMSGFRAME_DLG_WIDTH = 600;
    public static final int SHOWNMSGFRAME_DLG_HEIGHT = 250;

    // flag to set if only showing sections which have at least one key-value pair
    public static boolean shownAtLeastOneValueSection = false;

    // configuration sections
    public static String[] sections = {
                                "Gatekeeper::Main",
                                "Gatekeeper::Auth",
                                //"Gatekeeper::DestAnalysis",
                                "RoutedMode",
                                "Proxy",
                                //"Failover",
                                "RasSrv::RRQAuth",
                                "RasSvr::GWPrefixes",
                                //"RasSvr::ARQFeatures",
                                "RasSvr::RewriteE164",
                                "RasSvr::Neighbors",
                                "RasSvr::PermanentEndpoints",
                                "GkStatus::Auth",
                                "MySQLAuth",
                                "MySQLAliasAuth",
                                "SignalConnection::StatusEnquiry",
                                "Password",
                                "GkAuthorize",
                                "Endpoint",
                                "Endpoint::RewriteE164",
                                "CallTable",
                                //"GkLDAP::LDAPAttributeNames",
                                //"GkLDAP::Settings"
    };

    // each section's fixed options ...
    public static String[][] section_options = {
                        // Gatekeeper::Main
                        {
                            "Fourtytwo:2", "Name:1", "Home:1", "AlternateGKs:1", "sendto:1",
                            "SkipForwards:1", "EndpointIDSuffix:1", "UnicastRasPort:1", "MulticastPort:1",
                            "MulticastGroup:1", "EndpointSignalPort:1", "StatusPort:1", "ListenQueueLength:1",
                            "SignalReadTimeout:1", "StatusReadTimeout:1", "TimeToLive:1", "TotalBandwidth:1"
                        },
                        // Gatekeeper::Auth
                        {
                            "dynamic_option"
                        },
                        /*
                        // Gatekeeper::DestAnalysis
                        {
                            "dynamic_option"
                        },
                        */
                        // RoutedMode
                        {
                            "GKRouted:1", "H245Routed:1", "CallSignalPort:1", "CallSignalHandlerNumber:1",
                            "RemoveH245AddressOnTunneling:1", "AcceptNeighborsCalls:1", "AcceptUnregisteredCalls:1",
                            "SupportNATedEndpoints:1", "DropCallsByReleaseComplete:1"
                        },
                        // Proxy
                        {
                            "Enable:1", "InternalNetwork:1"
                        },
                        /*
                        // Failover
                        {
                            "dynamic_option"
                        },
                        */
                        // RasSrv::RRQAuth
                        {
                            "dynamic_option"
                        },
                        // RasSvr::GWPrefixes
                        {
                            "dynamic_option"
                        },
                        /*
                        // RasSvr::ARQFeatures
                        {
                            "dynamic_option"
                        },
                        */
                        // RasSvr::RewriteE164
                        {
                            "dynamic_option"
                        },
                        // RasSvr::Neighbors
                        {
                            "dynamic_option"
                        },
                        // RasSvr::PermanentEndpoints
                        {
                            "dynamic_option"
                        },
                        // GkStatus::Auth
                        {
                            "dynamic_option"
                        },
                        // MySQLAuth
                        {
                            "Host:1", "Database:1", "User:1", "Password:1", "Table:1", "IDField:1",
                            "PasswordField:1", "ExtraCriterion:1"
                        },
                        // MySQLAliasAuth
                        {
                            "Host:1", "Database:1", "User:1", "Password:1", "Table:1", "IDField:1",
                            "IPField:1", "ExtraCriterion:1"
                        },
                        // SignalConnection::StatusEnquiry
                        {
                            "UsePing:1", "AllowedPendings:1", "Timeout:1"
                        },
                        // Password
                        {
                            "KeyFilled:1", "CkeckID:1", "PasswordTimeout:1"
                        },
                        // GkAuthorize
                        {
                            "dynamic_option"
                        },
                        // Endpoint
                        {
                            "Gatekeeper:1", "Type:1", "H323ID:1", "E164:1", "Password:1",
                            "Prefix:1", "TimeToLive:1", "RRQRetryInterval:1", "ARQTimeout:1"
                        },
                        // Endpoint::RewriteE164
                        {
                            "dynamic_option"
                        },
                        // CallTable
                        {
                            "GenerateNBCDR:1", "GenerateUCCDR:1", "DefaultCallTimeout:1"
                        }
                        /*
                        // GkLDAP::LDAPAttributeNames
                        {
                            "dynamic_option"
                        },
                        */
                        /*
                        // GkLDAP::Settings
                        {
                            "dynamic_option"
                        }
                        */
    };

    // configuration sections usage ...
    public static String[] configUsages = {
                        // Gatekeeper::Main
                        " ## Boolean values.\n" +
                        " ## Boolean Values are retresented by a case insensitive string\n" +
                        " ## - \"t\"..., \"y\"... or \"1\" for TRUE\n" +
                        " ## - all other for FALSE\n\n" +
                        " ##\n" +
                        " ## Params used in  Gatekeeper::Main()\n" +
                        " ##\n" +
                        " ## NOTE: This parameters may be loaded at program startup and not influenced by the HUP signal. \n\n" +
                        " ## Example Setting and Explanations :\n\n" +
                        " [Gatekeeper::Main]\n" +
                        " # \'config is present\' indicator. Has to be 42.\n" +
                        " Fourtytwo=42\n" +
                        " # Includes in some RAS-Msgs\n" +
                        " Name=OpenH323GK\n" +
                        " # overwritten from command line parameter\n" +
                        " Home=195.71.129.69" +
                        " # TTL seting\n" +
                        " TimeToLive=600",

                        // Gatekeeper::Auth
                        " ## Boolean values.\n" +
                        " ## Boolean Values are retresented by a case insensitive string\n" +
                        " ## - \"t\"..., \"y\"... or \"1\" for TRUE\n" +
                        " ## - all other for FALSE\n\n" +
                        " ##\n" +
                        " ## Authentication mechanism\n" +
                        " ##\n" +
                        " ## Syntax:\n" +
                        " ##   authrule=actions\n" +
                        " ##\n" +
                        " ##   <authrule> := SimplePasswordAuth | LDAPPasswordAuth\n" +
                        " ##                 | AliasAuth | LDAPAliasAuth | ...\n" +
                        " ##   <actions>  := <control>[;<ras>,<ras>,...]\n" +
                        " ##   <control>  := optional | required | sufficient\n" +
                        " ##   <ras>      := GRQ | RRQ | URQ | ARQ | BRQ | DRQ | LRQ | IRQ\n" +
                        " ##\n" +
                        " ## Currently supported modules:\n" +
                        " ##\n" +
                        " ##   SimplePasswordAuth / LDAPPasswordAuth :\n" +
                        " ##                                                    The module checks the tokens or cryptoTokens\n" +
                        " ##                                                    fields of RAS message. The tokens should contain\n" +
                        " ##                                                    at least generalID and password. For cryptoTokens,\n" +
                        " ##                                                    cryptoEPPwdHash tokens hashed by simple MD5 and \n" +
                        " ##                                                    nestedcryptoToken tokens hashed by HMAC-SHA1-96\n" +
                        " ##                                                    (libssl must be installed!) are supported now.\n" +
                        " ##                                                    The ID and password are read from [Password] section\n" +
                        " ##                                                    / LDAP. Support for other backend databases is\n" +
                        " ##                                                    easily to add.\n" +
                        " ##\n" +
                        " ##   AliasAuth / LDAPAliasAuth :\n" +
                        " ##                                                    The IP of an endpoint with given alias should\n" +
                        " ##                                                    match a specified pattern. For AliasAuth the pattern\n" +
                        " ##                                                    is defined in [RasSrv::RRQAuth] section.\n" +
                        " ##                                                    For LDAPAliasAuth the alias (default: mail attribute)\n" +
                        " ##                                                    and IP (default: voIPIpAddress attribute) must be found\n" +
                        " ##                                                    in one LDAP entry.\n" +
                        " ##\n" +
                        " ## A rule may results in one of the three codes: ok, fail, pass.\n" +
                        " ##\n" +
                        " ##   ok :\n" +
                        " ##                                                    The request is authenticated by this module\n" +
                        " ##   fail :\n" +
                        " ##                                                    The authentication fails and should be rejected\n" +
                        " ##   next :\n" +
                        " ##                                                    The rule cannot determine the request\n" +
                        " ##\n" +
                        " ## There are also three ways to control a rule:\n" +
                        " ##\n" +
                        " ##   optional :\n" +
                        " ##                                                    If the rule cannot determine the request, it is passed\n" +
                        " ##                                                    to next rule.\n" +
                        " ##   required :\n" +
                        " ##                                                    The requests should be authenticated by this module,\n" +
                        " ##                                                    or it would be rejected. The authenticated request would \n" +
                        " ##                                                    then be passwd to next rule.\n" +
                        " ##   sufficient :\n" +
                        " ##                                                    If the request is authenticated, it is accepted,\n" +
                        " ##                                                    or it would be rejected. That is, the rule determines\n" +
                        " ##                                                    the fate of the request. No rule should be put after\n" +
                        " ##                                                    a sufficient rule, since it won't take effect.\n" +
                        " ##\n" +
                        " ## You can also configure a rule to check only for some particular RAS\n" +
                        " ## messages. For example, to configure SimplePasswordAuth as a required\n" +
                        " ## rule to check RRQ, ARQ and LRQ:\n" +
                        " ## SimplePasswordAuth=required;RRQ,ARQ,LRQ\n\n" +
                        " ## Example Setting and Explanations :\n\n" +
                        " [Gatekeeper::Auth]\n" +
                        " # SimplePasswordAuth=optional\n" +
                        " # LDAPPasswordAuth=optional\n" +
                        " # AliasAuth=sufficient;RRQ\n" +
                        " # LDAPAliasAuth=sufficient;RRQ\n" +
                        " # default=reject\n" +
                        " default=allow",

                        // Gatekeeper::DestAnalysis
                        /*
                        " ## Boolean values.\n" +
                        " ## Boolean Values are retresented by a case insensitive string\n" +
                        " ## - \"t\"..., \"y\"... or \"1\" for TRUE\n" +
                        " ## - all other for FALSE\n\n" +
                        " ##\n" +
                        " ## Destination analysis mechanism\n" +
                        " ##  (must be enabled with compiler option WITH_DEST_ANALYSIS_LIST)\n" +
                        " ##\n" +
                        " ## Syntax:\n" +
                        " ##   authrule=actions\n" +
                        " ##\n" +
                        " ##   <authrule> := OverlapSendDestAnalysis\n" +
                        " ##   <actions>  := <control>[;<message>,<message>,...]\n" +
                        " ##   <control>  := optional | required | sufficient\n" +
                        " ##   <message>  := ARQ | LRQ\n" +
                        " ##\n" +
                        " ## Currently supported modules:\n" +
                        " ##\n" +
                        " ##      OverlapSendDestAnalysis :\n" +
                        " ##                                                    This module checks for incomplete destination \n" +
                        " ##                                                    addresses (not fully implemented up to now).\n" +
                        " ##\n" +
                        " ## A rule may results in one of the three codes: ok, fail, pass.\n" +
                        " ## There are also three ways to control a rule: optional, required, sufficient.\n" +
                        " ## Additionally you can configure a rule to check only for some particular\n" +
                        " ## messages.\n" +
                        " ## (see Authentication mechanism for details informations).\n\n" +
                        " ## Example Setting and Explanations :\n\n" +
                        " [Gatekeeper::DestAnalysis]\n" +
                        " # OverlapSendDestAnalysis=required;ARQ\n" +
                        " # default=reject\n" +
                        " default=allow",
                        */

                        // GkStatus::Auth
                        " ## Boolean values.\n" +
                        " ## Boolean Values are retresented by a case insensitive string\n" +
                        " ## - \"t\"..., \"y\"... or \"1\" for TRUE\n" +
                        " ## - all other for FALSE\n\n" +
                        " ##\n" +
                        " ## The parameter \"rule\" may be one of the following :\n" +
                        " ## - \"forbid\" disallow any connection (default when no rule us given)\n" +
                        " ## - \"allow\" allow any connection\n" +
                        " ## - \"explicit\" reads the parameter #\"<ip>=<value>\"# with ip is the ip4-address\n" +
                        " ##   if the peering client. #<value># is resolved with #Toolkit::AsBool#. If the ip\n" +
                        " ##   is not listed the param \"default\" is used.\n" +
                        " ## - \"regex\" the #<ip># of the client is matched against the given regular expression. \n" +
                        " ##   First the ip-rules (like \"explicit\") are tested. Olny of no such param exists\n" +
                        " ##   the regex is tried.\n" +
                        " ##   Example: \"regex=^195\\.71\\.(129|131)\\.[0-9]+$\"\n\n" +
                        " ## Example Setting and Explanations :\n\n" +
                        " [GkStatus::Auth]\n" +
                        " rule=allow\n" +
                        " # rule=explicit\n" +
                        " # rule=regex\n" +
                        " # regex=^(195\\.71\\.(129|100)\\.[0-9]+)|(62\\.52\\.26\\.[1-2][0-9][0-9])$\n" +
                        " # default=forbid"
    };
}