package omnigkgui.gkcmd;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huagn, Hisao-Mei
 * @version      1.0
 */

public class GkCmdFindMsg extends ShownMsgFrame {
    // global objects
//    protected JScrollPane globalScrollPane = new JScrollPane();
//    protected JTextPane msgPane = new JTextPane();
    protected JPanel globalPane = new JPanel();
    protected JPanel epPane = new JPanel();
    protected JTextPane msgPane = new JTextPane();
    protected String originParams = null;


    // default constructor, declared "private" for prevent others to create
    // instances out-of-world by calling this method
     public GkCmdFindMsg() {
        originParams = "";

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public GkCmdFindMsg(String params, GkCommand commander) {
        super("GK Commands : For Showing Messages" + "  <" + commander.getHost() + ">");
        super.setCommander(commander);
        originParams = params;

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public GkCmdFindMsg(String title, String params, GkCommand commander) {
        super(title + "  <" + commander.getHost() + ">");
        super.setCommander(commander);
        originParams = params;

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // initialize function
    private void jbInit() throws Exception {
        Color bgColor = new Color(240, 223, 223);
        // modify the root frame to suitable size
        super.setSize(GkCmdConstants.SHOWNMSGFRAME_DLG_WIDTH, GkCmdConstants.SHOWNMSGFRAME_DLG_HEIGHT);
        super.validate();

        // set the globalPane size
        Dimension preferedPaneSize = super.getRootPane().getSize();
        globalPane.setSize(preferedPaneSize);
        globalPane.setBackground(bgColor);

        // set the LayoutManager of global pane
        globalPane.setLayout(new BorderLayout());

        // set the endpoint pane parameters
        epPane.setSize(GkCmdConstants.SHOWNMSGFRAME_DLG_WIDTH / 2, GkCmdConstants.SHOWNMSGFRAME_DLG_HEIGHT / 2);
        epPane.setBorder(new LineBorder(Color.blue, 3, true));
        epPane.setBackground(Color.white);

        // set the message pane parameters and message font
        msgPane.setBackground(bgColor);
        msgPane.setFont(new Font("Helvetica", Font.PLAIN, 16));
        msgPane.setEditable(false);

        globalPane.add(epPane, BorderLayout.NORTH);
        globalPane.add(msgPane, BorderLayout.CENTER);
        super.getContentPane().add(globalPane, null);

        // process the input parameters
        if ((originParams != null) && (!originParams.equals(""))) {
            initStylesForMsgPane(msgPane);
            parseParams(originParams);
        }
    }

    /* private methods for internal use */
    private void initStylesForMsgPane(JTextPane msgPane) {
        //Initialize some styles.
        Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);

        Style regular = msgPane.addStyle("regular", def);

        Style s = msgPane.addStyle("cmd", regular);
        StyleConstants.setFontSize(s, 18);
        StyleConstants.setBold(s, true);
        StyleConstants.setBackground(s, new Color(100, 145, 215));
        StyleConstants.setForeground(s, Color.orange);
        /*
        StyleConstants.setAlignment(s, StyleConstants.ALIGN_CENTER);
        JButton button = new JButton(" Hi ... ");
        button.setMargin(new Insets(2, 5, 2, 5));
        button.setFont(new Font("Helvetica", Font.BOLD, 20));
        button.setBackground(new Color(100, 145, 215));
        button.setForeground(Color.orange);
        StyleConstants.setComponent(s, button);
        */

        s = msgPane.addStyle("usage", regular);
        StyleConstants.setFontFamily(s, "Currier New");
        StyleConstants.setItalic(s, true);
        StyleConstants.setForeground(s, new Color(85, 175, 80));
    }

    /** abstract methods implementation */
    protected void parseParams(String params) {
        String indent = "        ";
        String newline = "\n";
        String delimiter = "\n\t";
        StringTokenizer reader = new StringTokenizer(params, delimiter);
        String token = null;

        Document d = msgPane.getDocument();

        JButton ep = new JButton("Test");
        epPane.add(ep);


        while (reader.hasMoreTokens()) {
            token = reader.nextToken();

            if (token.startsWith("RCF")) {
                try {
                    // insert cmd message
                    d.insertString(d.getLength(), token, msgPane.getStyle("usage"));
                    d.insertString(d.getLength(), newline, msgPane.getStyle("usage"));
                } catch (BadLocationException ble) {
                    // just ignore this exception
                }
            } else {
                try {
                    d.insertString(d.getLength(), token, msgPane.getStyle("regular"));
                    d.insertString(d.getLength(), newline, msgPane.getStyle("regular"));
                } catch (BadLocationException ble) {
                    // just ignore this exception
                }
            }
        }
    }

    public String getOriginalParams() {
        return originParams;
    }
}