package omnigkgui.gkcmd;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huagn, Hisao-Mei
 * @version      1.0
 */

public class GkCmdFindVerboseMsg extends ShownMsgFrame {
    // global objects
    protected JScrollPane globalScrollPane = new JScrollPane();
    protected JTextPane msgPane = new JTextPane();
    protected String originParams = null;

    // default constructor, declared "private" for prevent others to create
    // instances out-of-world by calling this method
     public GkCmdFindVerboseMsg() {
        originParams = "";

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public GkCmdFindVerboseMsg(String params, GkCommand commander) {
        super("GK Commands : For Showing Messages" + "  <" + commander.getHost() + ">");
        super.setCommander(commander);
        originParams = params;

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public GkCmdFindVerboseMsg(String title, String params, GkCommand commander) {
        super(title + "  <" + commander.getHost() + ">");
        super.setCommander(commander);
        originParams = params;

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // initialize function
    private void jbInit() throws Exception {
        // set the globalScrollPane size
        Dimension preferedPaneSize = super.getRootPane().getSize();
        globalScrollPane.setSize(preferedPaneSize);
        // set globalScrollPane background color
        globalScrollPane.setBackground(SystemColor.info);

        // set the message pane parameters and message font
        msgPane.setBackground(SystemColor.info);
        msgPane.setFont(new Font("Helvetica", Font.PLAIN, 14));
        msgPane.setEditable(false);

        globalScrollPane.setViewportView(msgPane);
        super.getContentPane().add(globalScrollPane, null);

        // process the input parameters
        if ((originParams != null) && (!originParams.equals(""))) {
            initStylesForMsgPane(msgPane);
            parseParams(originParams);
        }
    }

    /* private methods for internal use */
    private void initStylesForMsgPane(JTextPane msgPane) {
        //Initialize some styles.
        Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);

        Style regular = msgPane.addStyle("regular", def);

        Style s = msgPane.addStyle("cmd", regular);
        StyleConstants.setFontSize(s, 18);
        StyleConstants.setBold(s, true);
        StyleConstants.setBackground(s, new Color(100, 145, 215));
        StyleConstants.setForeground(s, Color.orange);
        /*
        StyleConstants.setAlignment(s, StyleConstants.ALIGN_CENTER);
        JButton button = new JButton(" Hi ... ");
        button.setMargin(new Insets(2, 5, 2, 5));
        button.setFont(new Font("Helvetica", Font.BOLD, 20));
        button.setBackground(new Color(100, 145, 215));
        button.setForeground(Color.orange);
        StyleConstants.setComponent(s, button);
        */

        s = msgPane.addStyle("usage", regular);
        StyleConstants.setFontFamily(s, "Currier New");
        StyleConstants.setItalic(s, true);
        StyleConstants.setForeground(s, new Color(85, 175, 80));
    }

    /** abstract methods implementation */
    protected void parseParams(String params) {
        String indent = "        ";
        String newline = "\n";
        String delimiter = "\n\t";
        StringTokenizer reader = new StringTokenizer(params, delimiter);
        String token = null;

        Document d = msgPane.getDocument();

        while (reader.hasMoreTokens()) {
            token = reader.nextToken();

            if (token.startsWith("RCF")) {
                try {
                    // insert cmd message
                    d.insertString(d.getLength(), token, msgPane.getStyle("usage"));
                    d.insertString(d.getLength(), newline, msgPane.getStyle("usage"));
                } catch (BadLocationException ble) {
                    // just ignore this exception
                }
            } else {
                try {
                    d.insertString(d.getLength(), token, msgPane.getStyle("regular"));
                    d.insertString(d.getLength(), newline, msgPane.getStyle("regular"));
                } catch (BadLocationException ble) {
                    // just ignore this exception
                }
            }
        }
    }

    public String getOriginalParams() {
        return originParams;
    }
}