package omnigkgui.gkcmd;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;
import javax.swing.tree.*;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huagn, Hisao-Mei
 * @version      1.0
 */

public class GkCmdHelpMsg extends ShownMsgFrame {
    // global objects
    protected JScrollPane globalScrollPane = new JScrollPane();
    protected JPanel contentPane = new JPanel();
    protected JTree cmdTree = new JTree();
    protected JTextPane msgPane = new JTextPane();
    protected String originParams = null;

    // cmdTree default nodes
    DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("GK Commands");
    DefaultMutableTreeNode endpointNode = new DefaultMutableTreeNode("Class for <Endpoint>");
    DefaultMutableTreeNode callNode = new DefaultMutableTreeNode("Class for <Call>");
    DefaultMutableTreeNode operationNode = new DefaultMutableTreeNode("Class for <Operation>");
    DefaultMutableTreeNode miscNode = new DefaultMutableTreeNode("Class for <Miscellaneous>");

    // static command strings
    static String[] cmdEndpointStr = { "printallregistrations", "printallregistrationsverbose",
                                "r", "rv", "?", "??" };
    static String[] cmdCallStr = { "printcurrentcalls", "printcurrentcallsverbose",
                                "c", "cv", "!", "!!" };
    static String[] cmdOperationStr = { "find", "f", "findverbose", "fv", "disconnectip", "disconnectcall",
                                "disconnectalias", "disconnectendpoint", "unregisterallendpoints",
                                "unregisteralias", "unregisterip", "makecall", "reload" };

    // default constructor, declared "private" for prevent others to create
    // instances out-of-world by calling this method
     public GkCmdHelpMsg() {
        originParams = "";

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public GkCmdHelpMsg(String params, GkCommand commander) {
        super("GK Commands : For Showing Messages" + "  <" + commander.getHost() + ">");
        super.setCommander(commander);
        originParams = params;

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public GkCmdHelpMsg(String title, String params, GkCommand commander) {
        super(title + "  <" + commander.getHost() + ">");
        super.setCommander(commander);
        originParams = params;

        try {
            jbInit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // initialize function
    private void jbInit() throws Exception {
        // set the globalScrollPane size
        Dimension preferedPaneSize = super.getRootPane().getSize();
        globalScrollPane.setSize(preferedPaneSize);
        // set globalScrollPane background color
        globalScrollPane.setBackground(SystemColor.info);

        // set the content pane parameters
        contentPane.setLayout(new BorderLayout());

        // set the cmd tree parameters
        cmdTree.setBorder(new LineBorder(Color.orange, 5, true));
        cmdTree.setFont(new Font("Helvetica", Font.BOLD, 14));
        // add cmd category into root node
        rootNode.add(endpointNode);
        rootNode.add(callNode);
        rootNode.add(operationNode);
        rootNode.add(miscNode);
        DefaultTreeModel treeModel = new DefaultTreeModel(rootNode);
        cmdTree.setModel(treeModel);

        // set the message pane parameters and message font
        msgPane.setBackground(SystemColor.info);
        msgPane.setFont(new Font("Helvetica", Font.PLAIN, 14));
        //msgPane.setBorder(new LineBorder(Color.blue, 3, true));
        msgPane.setEditable(false);

        contentPane.add(cmdTree, BorderLayout.NORTH);
        contentPane.add(msgPane, BorderLayout.SOUTH);
//      globalScrollPane.setViewportView(msgPane);
        globalScrollPane.setViewportView(contentPane);
        super.getContentPane().add(globalScrollPane, null);

        // process the input parameters
        if ((originParams != null) && (!originParams.equals(""))) {
            initStylesForMsgPane(msgPane);
            parseParams(originParams);
        }
    }

    /* private methods for internal use */
    private void initStylesForMsgPane(JTextPane msgPane) {
        //Initialize some styles.
        Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);

        Style regular = msgPane.addStyle("regular", def);

        Style s = msgPane.addStyle("cmd", regular);
        StyleConstants.setFontSize(s, 18);
        StyleConstants.setBold(s, true);
        StyleConstants.setBackground(s, new Color(100, 145, 215));
        StyleConstants.setForeground(s, Color.orange);
        /*
        StyleConstants.setAlignment(s, StyleConstants.ALIGN_CENTER);
        JButton button = new JButton(" Hi ... ");
        button.setMargin(new Insets(2, 5, 2, 5));
        button.setFont(new Font("Helvetica", Font.BOLD, 20));
        button.setBackground(new Color(100, 145, 215));
        button.setForeground(Color.orange);
        StyleConstants.setComponent(s, button);
        */

        s = msgPane.addStyle("usage", regular);
        StyleConstants.setFontFamily(s, "Currier New");
        StyleConstants.setItalic(s, true);
        StyleConstants.setForeground(s, new Color(85, 175, 80));
    }


    /** abstract methods implementation */
    protected void parseParams(String params) {
        String indent = "        ";
        String newline = "\n";
        String delimiter = "\n\t";
        StringTokenizer reader = new StringTokenizer(params, delimiter);
        String token = null;

        /*
        SimpleAttributeSet cmdAttribute = new SimpleAttributeSet();
        StyleConstants.setFontFamily(cmdAttribute, "Courier New");
        StyleConstants.setForeground(cmdAttribute, Color.red);
        StyleConstants.setFontSize(cmdAttribute, 18);
        StyleConstants.setBold(cmdAttribute, true);
        SimpleAttributeSet usageAttribute = new SimpleAttributeSet();
        StyleConstants.setForeground(usageAttribute, Color.cyan);
        StyleConstants.setFontSize(usageAttribute, 14);
        StyleConstants.setItalic(usageAttribute, true);
        */

        Document d = msgPane.getDocument();

        while (reader.hasMoreTokens()) {
            token = reader.nextToken();

            if (!token.equals("Commands:")) {
//              Color originForegroundColor = msgPane.getForeground();
//              msgPane.setForeground(Color.red);
//              JTextArea cmd = new JTextArea(indent + token + " :\n");
//              cmd.setBackground(msgPane.getBackground());
//              cmd.setForeground(Color.red);
//              msgPane.insertComponent(cmd);
//              msgPane.setForeground(originForegroundColor);
//              JTextArea usage = new JTextArea(indent + indent + "Explanation Command usages here !!\n\n");
//              usage.setBackground(msgPane.getBackground());
//              usage.setForeground(Color.magenta);
//              msgPane.insertComponent(usage);
                try {
                    // insert cmd into cmd tree
                    //((DefaultMutableTreeNode)cmdTree.getModel().getRoot()).add(new DefaultMutableTreeNode(token));
                    int category = getCmdCategory(token);
                    switch (category) {
                        case 0 :    // 0 is belong to Class <Endpoint>
                                endpointNode.add(new DefaultMutableTreeNode(token));
                                break;
                        case 1 :    // 1 is belong to Class <Call>
                                callNode.add(new DefaultMutableTreeNode(token));
                                break;
                        case 2 :    // 2 is belong to Class <Operation>
                                operationNode.add(new DefaultMutableTreeNode(token));
                                break;
                        default :
                                miscNode.add(new DefaultMutableTreeNode(token));
                    }
                    //miscNode.add(new DefaultMutableTreeNode(token));

                    // insert cmd message
                    d.insertString(d.getLength(), " ", msgPane.getStyle("regular"));
                    JButton cmdBtn = new JButton(" " + token + " ");
                    cmdBtn.setFont(new Font("Helvetica", Font.BOLD, 18));
                    cmdBtn.setBackground(new Color(100, 145, 215));
                    cmdBtn.setForeground(Color.orange);
                    msgPane.insertComponent(cmdBtn);
                    d.insertString(d.getLength(), newline, msgPane.getStyle("regular"));

                    d.insertString(d.getLength(), indent, msgPane.getStyle("usage"));
                    String cmdStr = " " + token + " ";
                    d.insertString(d.getLength(), cmdStr, msgPane.getStyle("cmd"));
                    d.insertString(d.getLength(), newline + newline, msgPane.getStyle("regular"));
                    String usageStr = indent + indent + "Explanation Command usages here !!\n\n";
                    d.insertString(d.getLength(), usageStr, msgPane.getStyle("usage"));
                } catch (BadLocationException ble) {
                    // just ignore this exception
                }
            }
        }
        cmdTree.expandRow(0);
    }

    public String getOriginalParams() {
        return originParams;
    }

    private int getCmdCategory(String cmd) {
        // search for first command strings category
        for (int i = 0; i < cmdEndpointStr.length; i++) {
            if (cmd.equals(cmdEndpointStr[i]))
                return 0;
        }

        // search for second command strings category
        for (int j = 0; j < cmdCallStr.length; j++) {
            if (cmd.equals(cmdCallStr[j]))
                return 1;
        }

        // search for third command strings category
        for (int k = 0; k < cmdOperationStr.length; k++) {
            if (cmd.equals(cmdOperationStr[k]))
                return 2;
        }

        // not find in above categories, just miscellaneous commands
        return 4;
    }
}