package omnigkgui.gkcmd;

import javax.swing.*;
import java.net.*;    // Socket
import java.io.*;     // IOException
import java.sql.*;

import omnigkgui.*;
import javax.swing.border.*;
import java.awt.*;

/**
 * Title:        Citron Omni GateKeeper Graphic User Interface Version 1.0
 * Description:  This is the Citron's GateKeeper's user interface.
 *               Its purpose is to achieve using one interface, running everywhere !!
 * Copyright:    Copyright (c) 2001
 * Company:      Citron Inc.
 * @author       Kau, Kuang-Jiue; Huagn, Hisao-Mei
 * @version      1.0
 */

public class GkCommand implements Runnable, GkCmdIDs {
    private GateKeeper gk;
    private String host;
    private int port;

    // socket io to communicate with the gatekeeper
    private Socket socket = null;
    private InputStream istream = null;
    private OutputStream ostream = null;

    // piped io to transfer character-stream between threads
    // protected PipedReader pipedInStream = null;  // in-pipe for others to write something in ...
    protected PipedWriter pipedOutStream = null;  // out-pipe to write something to others ...

    // GK command flag
    private int gkCmdFlag = 0;

    /** Default Constructor */
    public GkCommand() {

    }

    public void init(GateKeeper _gk, String _host, int _port) {
        gk = _gk;
        host = _host;
        port = _port;
    }

    public void stop() {
        try {
            // cleanup
            if (istream != null)
                istream.close();
            istream = null;
            if (ostream != null)
                ostream.close();
            ostream = null;
            if (socket != null)
                socket.close();
            socket = null;
        } catch (IOException e) {
            return;
        }
    }

    public void run() {
        if (istream != null) {
            return;
        }

        try {
//          gk.appendLog("Commander connecting to gatekeeper at " + host + ":" + port + "\n");
            socket = new Socket(host, port);
        } catch (Exception e) {
            gk.appendLog("Error: " + e + "\n");
            return;
        }

        try {
            ostream = socket.getOutputStream();
            istream = socket.getInputStream();
            BufferedReader in = new BufferedReader(new InputStreamReader(istream));

//          ostream.write("PrintAllRegistrations\n".getBytes());
//          ostream.write("PrintCurrentCalls\n".getBytes());

            String inputLine;
            String result = "";
            while ((inputLine = in.readLine()) != null) {
                if ((gkCmdFlag == 0) || (inputLine.endsWith(";") && !inputLine.startsWith(";"))) {
                    // just ignore, do nothing
                } else {
                    switch (gkCmdFlag) {
                        case GkCmdIDs.GKCmdFind :
                            if (inputLine.startsWith("SoftPBX")) {
                                // error : alias not find
                                setGkCmdFlag(0);
                                GkCmdFindMsg shownFrame = new GkCmdFindMsg("GK Commands : Find", inputLine, this);
                            } else if (!inputLine.startsWith(";")) {
                                // start with ";" is the end of GK command session
                                result += inputLine + "\n";
                            } else {
                                setGkCmdFlag(0);
                                GkCmdFindMsg shownFrame = new GkCmdFindMsg("GK Commands : Find", result, this);
                                result = "";
                            }
                            break;

                        case GkCmdIDs.GKCmdFindVerbose :
                            if (inputLine.startsWith("SoftPBX")) {
                                // error : alias not find
                                setGkCmdFlag(0);
                                GkCmdFindVerboseMsg shownFrame = new GkCmdFindVerboseMsg("GK commands : Find Verbose", inputLine, this);
                            } else if (!inputLine.startsWith(";")) {
                                // start with ";" is the end of GK command session
                                result += inputLine + "\n";
                            } else {
                                setGkCmdFlag(0);
                                GkCmdFindVerboseMsg shownFrame = new GkCmdFindVerboseMsg("GK Commands : Find Verbose", result, this);
                                result = "";
                            }
                            break;

                        case GkCmdIDs.GKCmdHelp :
                            if (!inputLine.startsWith(";")) {
                                // start with ";" is the end of GK command session
                                result += inputLine + "\n";
                            } else {
                                setGkCmdFlag(0);
                                GkCmdHelpMsg shownFrame = new GkCmdHelpMsg("GK Commands : Help", result, this);
                                result = "";
                            }
                            break;

                        case GkCmdIDs.GKCmdDebug :
                            if (inputLine.startsWith(";")) {
                                // exit the section
                                pipedOutStream.write("EOF_STREAM\n");
                                //setGkCmdFlag(0);
                                //pipedOutStream.close();
                            } else {
                                // write the received stream from gatekeeper to another
                                // processor by piped-outstream
                                pipedOutStream.write(inputLine + "\n");
                            }
                            break;

                        default:
                        // just ignore, do nothing
                    }
                }
            }
            stop();
        } catch (IOException ioe) {
            return;
        }
        // app.appendLog("Client thread stopped\n");
    }

    // extract a field (counting from one) from a pipe-seperated string,
    // format --> ("header |field 1| field 2|field 3")
    protected String extractField(String _msg, int _which) {
        String token = "";
        int start = 0, end = 0;

        if (_msg.endsWith(";"))
            _msg = _msg.substring(0, _msg.length() - 1);
        _msg += "|";

        for (int i = 0; i < _which; i++)
            start = _msg.indexOf("|", start) + 1;
        end = _msg.indexOf("|", start);

        try {
            token = _msg.substring(start, end);
        } finally {
            return token;
        }
    }

    // set GK command flag to reflect which command is in action
    synchronized protected void setGkCmdFlag(int cmdID) {
        this.gkCmdFlag = cmdID;
    }

    // transfer GK command to GK
    public void transferCommand(int cmdID, String params) {
        try {
            // commands need no parameters
            if (params == null) {
                if (gkCmdFlag == 0) {
                    switch (cmdID) {
                        case GkCmdIDs.GKCmdHelp :
                            setGkCmdFlag(GkCmdIDs.GKCmdHelp);
                            ostream.write((new String("Help\n")).getBytes());
                            break;

                        case GkCmdIDs.GKCmdDebug :
                            setGkCmdFlag(GkCmdIDs.GKCmdDebug);
//                          GkCmdConfigMsg shownFrame = new GkCmdConfigMsg(istream, ostream, this);
                            pipedOutStream = new PipedWriter();
                            new Thread(new GkCmdConfigMsg(pipedOutStream, ostream, this)).start();
//                          setGkCmdFlag(0);
                            break;
                    }
                }
            } else {    // commands need some parameters
                if (gkCmdFlag == 0) {
                    switch (cmdID) {
                        case GkCmdIDs.GKCmdFind :
                            setGkCmdFlag(GkCmdIDs.GKCmdFind);
                            ostream.write(("Find " + params + "\n").getBytes());
                            break;

                        case GkCmdIDs.GKCmdFindVerbose :
                            setGkCmdFlag(GkCmdIDs.GKCmdFindVerbose);
                            ostream.write(("FindVerbose " + params + "\n").getBytes());
                            break;
                    }
                }
            }
        } catch (IOException e) {
            gk.appendLog("Error: " + e + "\n");
            return;
        }
    }

    // data member accessors
    protected String getHost() {
        return this.host;
    }
}
